@@ -46,67 +46,59 @@
 #include "DefaultDiskWriterFactory.h"
 #include "DlAbortEx.h"
 #include "File.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "SimpleRandomizer.h"
 
 namespace aria2 {
 
-DiskWriterEntry::DiskWriterEntry(const SharedHandle<FileEntry>& fileEntry):
-  fileEntry(fileEntry), _open(false), _directIO(false),
-  _needsFileAllocation(false) {}
-
-DiskWriterEntry::~DiskWriterEntry() {}
+DiskWriterEntry::DiskWriterEntry(const SharedHandle<FileEntry>& fileEntry)
+  : fileEntry_(fileEntry),
+    open_(false),
+    needsFileAllocation_(false)
+{}
 
 const std::string& DiskWriterEntry::getFilePath() const
 {
-  return fileEntry->getPath();
+  return fileEntry_->getPath();
 }
 
 void DiskWriterEntry::initAndOpenFile()
 {
-  if(!diskWriter.isNull()) {
-    diskWriter->initAndOpenFile(fileEntry->getLength());
-    if(_directIO) {
-      diskWriter->enableDirectIO();
-    }
-    _open = true;
+  if(diskWriter_) {
+    diskWriter_->initAndOpenFile(fileEntry_->getLength());
+    open_ = true;
   }
 }
 
 void DiskWriterEntry::openFile()
 {
-  if(!diskWriter.isNull()) {
-    diskWriter->openFile(fileEntry->getLength());
-    if(_directIO) {
-      diskWriter->enableDirectIO();
-    }
-    _open = true;
+  if(diskWriter_) {
+    diskWriter_->openFile(fileEntry_->getLength());
+    open_ = true;
   }
 }
 
 void DiskWriterEntry::openExistingFile()
 {
-  if(!diskWriter.isNull()) {
-    diskWriter->openExistingFile(fileEntry->getLength());
-    if(_directIO) {
-      diskWriter->enableDirectIO();
-    }
-    _open = true;
+  if(diskWriter_) {
+    diskWriter_->openExistingFile(fileEntry_->getLength());
+    open_ = true;
   }
 }
 
 void DiskWriterEntry::closeFile()
 {
-  if(_open) {
-    diskWriter->closeFile();
-    _open = false;
+  if(open_) {
+    diskWriter_->closeFile();
+    open_ = false;
   }
 }
 
 bool DiskWriterEntry::fileExists()
 {
-  return fileEntry->exists();
+  return fileEntry_->exists();
 }
 
 uint64_t DiskWriterEntry::size() const
@@ -116,70 +108,54 @@ uint64_t DiskWriterEntry::size() const
 
 void DiskWriterEntry::setDiskWriter(const SharedHandle<DiskWriter>& diskWriter)
 {
-  this->diskWriter = diskWriter;
+  diskWriter_ = diskWriter;
 }
 
 bool DiskWriterEntry::operator<(const DiskWriterEntry& entry) const
 {
-  return fileEntry < entry.fileEntry;
+  return *fileEntry_ < *entry.fileEntry_;
 }
 
-void DiskWriterEntry::enableDirectIO()
-{
-  if(_open) {
-    diskWriter->enableDirectIO();
-  }
-  _directIO = true;
-}
-
-void DiskWriterEntry::disableDirectIO()
-{
-  if(_open) {
-    diskWriter->disableDirectIO();
-  }
-  _directIO = false;
-}
-
-MultiDiskAdaptor::MultiDiskAdaptor():
-  pieceLength(0),
-  _maxOpenFiles(DEFAULT_MAX_OPEN_FILES),
-  _directIOAllowed(false),
-  _readOnly(false) {}
+MultiDiskAdaptor::MultiDiskAdaptor()
+  : pieceLength_(0),
+    maxOpenFiles_(DEFAULT_MAX_OPEN_FILES),
+    readOnly_(false)
+{}
 
 MultiDiskAdaptor::~MultiDiskAdaptor() {}
 
-static SharedHandle<DiskWriterEntry> createDiskWriterEntry
+namespace {
+SharedHandle<DiskWriterEntry> createDiskWriterEntry
 (const SharedHandle<FileEntry>& fileEntry,
  bool needsFileAllocation)
 {
   SharedHandle<DiskWriterEntry> entry(new DiskWriterEntry(fileEntry));
   entry->needsFileAllocation(needsFileAllocation);
   return entry;
-}
- 
+} 
+} // namespace
 
 void MultiDiskAdaptor::resetDiskWriterEntries()
 {
-  diskWriterEntries.clear();
+  diskWriterEntries_.clear();
 
-  if(fileEntries.empty()) {
+  if(getFileEntries().empty()) {
     return;
   }
 
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-        fileEntries.begin(); i != fileEntries.end(); ++i) {
-    diskWriterEntries.push_back
+        getFileEntries().begin(), eoi = getFileEntries().end(); i != eoi; ++i) {
+    diskWriterEntries_.push_back
       (createDiskWriterEntry(*i, (*i)->isRequested()));
   }
-
   std::map<std::string, bool> dwreq;
-
-  // TODO Currently, pieceLength == 0 is used for unit testing only.
-  if(pieceLength > 0) {
-    std::vector<SharedHandle<DiskWriterEntry> >::iterator done =
-      diskWriterEntries.begin();
-    for(std::vector<SharedHandle<DiskWriterEntry> >::iterator itr =
-          diskWriterEntries.begin(); itr != diskWriterEntries.end();) {
+  // TODO Currently, pieceLength_ == 0 is used for unit testing only.
+  if(pieceLength_ > 0) {
+    std::vector<SharedHandle<DiskWriterEntry> >::const_iterator done =
+      diskWriterEntries_.begin();
+    for(std::vector<SharedHandle<DiskWriterEntry> >::const_iterator itr =
+          diskWriterEntries_.begin(), eoi = diskWriterEntries_.end();
+        itr != eoi;) {
       const SharedHandle<FileEntry>& fileEntry = (*itr)->getFileEntry();
 
       if(!fileEntry->isRequested()) {
@@ -187,9 +163,9 @@ void MultiDiskAdaptor::resetDiskWriterEntries()
         continue;
       }
       off_t pieceStartOffset =
-        (fileEntry->getOffset()/pieceLength)*pieceLength;
-      if(itr != diskWriterEntries.begin()) {
-        for(std::vector<SharedHandle<DiskWriterEntry> >::iterator i =
+        (fileEntry->getOffset()/pieceLength_)*pieceLength_;
+      if(itr != diskWriterEntries_.begin()) {
+        for(std::vector<SharedHandle<DiskWriterEntry> >::const_iterator i =
               itr-1; true; --i) {
           const SharedHandle<FileEntry>& fileEntry = (*i)->getFileEntry();
           if(pieceStartOffset <= fileEntry->getOffset() ||
@@ -207,26 +183,27 @@ void MultiDiskAdaptor::resetDiskWriterEntries()
 
       if(fileEntry->getLength() > 0) {
         off_t lastPieceStartOffset =
-          (fileEntry->getOffset()+fileEntry->getLength()-1)/pieceLength*pieceLength;
-        logger->debug("Checking adjacent backward file to %s"
-                      " whose lastPieceStartOffset+pieceLength=%lld",
-                      fileEntry->getPath().c_str(),
-                      lastPieceStartOffset+pieceLength);
-
+          (fileEntry->getOffset()+fileEntry->getLength()-1)/
+          pieceLength_*pieceLength_;
+        A2_LOG_DEBUG(fmt("Checking adjacent backward file to %s"
+                         " whose lastPieceStartOffset+pieceLength_=%lld",
+                         fileEntry->getPath().c_str(),
+                         static_cast<long long int>
+                         (lastPieceStartOffset+pieceLength_)));
         ++itr;
         // adjacent backward files are not needed to be allocated. They
         // just requre DiskWriter
-        for(; itr != diskWriterEntries.end() &&
+        for(; itr != eoi &&
               (!(*itr)->getFileEntry()->isRequested() ||
                (*itr)->getFileEntry()->getLength() == 0); ++itr) {
-          logger->debug("file=%s, offset=%lld",
-                        (*itr)->getFileEntry()->getPath().c_str(),
-                        (*itr)->getFileEntry()->getOffset());
-
+          A2_LOG_DEBUG(fmt("file=%s, offset=%lld",
+                           (*itr)->getFileEntry()->getPath().c_str(),
+                           static_cast<long long int>
+                           ((*itr)->getFileEntry()->getOffset())));
           if((*itr)->getFileEntry()->getOffset() <
-             static_cast<off_t>(lastPieceStartOffset+pieceLength)) {
-            logger->debug("%s needs diskwriter",
-                          (*itr)->getFileEntry()->getPath().c_str());
+             static_cast<off_t>(lastPieceStartOffset+pieceLength_)) {
+            A2_LOG_DEBUG(fmt("%s needs diskwriter",
+                             (*itr)->getFileEntry()->getPath().c_str()));
             dwreq[(*itr)->getFileEntry()->getPath()] = true;
           } else {
             break;
@@ -240,67 +217,60 @@ void MultiDiskAdaptor::resetDiskWriterEntries()
     }
   }
   DefaultDiskWriterFactory dwFactory;
-  for(std::vector<SharedHandle<DiskWriterEntry> >::iterator i =
-        diskWriterEntries.begin(); i != diskWriterEntries.end(); ++i) {
+  for(std::vector<SharedHandle<DiskWriterEntry> >::const_iterator i =
+        diskWriterEntries_.begin(), eoi = diskWriterEntries_.end();
+      i != eoi; ++i) {
     if((*i)->needsFileAllocation() ||
        dwreq.find((*i)->getFileEntry()->getPath()) != dwreq.end() ||
        (*i)->fileExists()) {
-      logger->debug("Creating DiskWriter for filename=%s",
-                    (*i)->getFilePath().c_str());
+      A2_LOG_DEBUG(fmt("Creating DiskWriter for filename=%s",
+                       (*i)->getFilePath().c_str()));
       (*i)->setDiskWriter(dwFactory.newDiskWriter((*i)->getFilePath()));
-      if(_directIOAllowed) {
-        (*i)->getDiskWriter()->allowDirectIO();
-      }
-      if(_readOnly) {
+      if(readOnly_) {
         (*i)->getDiskWriter()->enableReadOnly();
       }
     }
   }
 }
 
-void MultiDiskAdaptor::mkdir() const
-{
-  for(std::vector<SharedHandle<DiskWriterEntry> >::const_iterator i =
-        diskWriterEntries.begin(); i != diskWriterEntries.end(); ++i) {
-    (*i)->getFileEntry()->setupDir();
-  }
-}
-
 void MultiDiskAdaptor::openIfNot
 (const SharedHandle<DiskWriterEntry>& entry, void (DiskWriterEntry::*open)())
 {
   if(!entry->isOpen()) {
-    //     logger->debug("DiskWriterEntry: Cache MISS. offset=%s",
-    //            util::itos(entry->getFileEntry()->getOffset()).c_str());
+    //     A2_LOG_DEBUG(fmt("DiskWriterEntry: Cache MISS. offset=%s",
+    //            util::itos(entry->getFileEntry()->getOffset()).c_str()));
  
-    size_t numOpened = _openedDiskWriterEntries.size();
+    size_t numOpened = openedDiskWriterEntries_.size();
     (entry.get()->*open)();
-    if(numOpened >= _maxOpenFiles) {
+    if(numOpened >= maxOpenFiles_) {
       // Cache is full. 
       // Choose one DiskWriterEntry randomly and close it.
-      size_t index = SimpleRandomizer::getInstance()->getRandomNumber(numOpened);
+      size_t index =
+        SimpleRandomizer::getInstance()->getRandomNumber(numOpened);
       std::vector<SharedHandle<DiskWriterEntry> >::iterator i =
-        _openedDiskWriterEntries.begin();
+        openedDiskWriterEntries_.begin();
       std::advance(i, index);
       (*i)->closeFile();
       (*i) = entry;
     } else {
-      _openedDiskWriterEntries.push_back(entry);
+      openedDiskWriterEntries_.push_back(entry);
     } 
   } else {
-    //     logger->debug("DiskWriterEntry: Cache HIT. offset=%s",
-    //            util::itos(entry->getFileEntry()->getOffset()).c_str());
+    //     A2_LOG_DEBUG(fmt("DiskWriterEntry: Cache HIT. offset=%s",
+    //            util::itos(entry->getFileEntry()->getOffset()).c_str()));
   }
 }
 
 void MultiDiskAdaptor::openFile()
 {
   resetDiskWriterEntries();
-  mkdir();
+  // util::mkdir() is called in AbstractDiskWriter::createFile(), so
+  // we don't need to call it here.
+
   // Call DiskWriterEntry::openFile to make sure that zero-length files are
   // created.
-  for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
-      itr != diskWriterEntries.end(); ++itr) {
+  for(DiskWriterEntries::const_iterator itr = diskWriterEntries_.begin(),
+        eoi = diskWriterEntries_.end(); itr != eoi; ++itr) {
     openIfNot(*itr, &DiskWriterEntry::openFile);
   }
 }
@@ -308,10 +278,12 @@ void MultiDiskAdaptor::openFile()
 void MultiDiskAdaptor::initAndOpenFile()
 {
   resetDiskWriterEntries();
-  mkdir();
+  // util::mkdir() is called in AbstractDiskWriter::createFile(), so
+  // we don't need to call it here.
+
   // Call DiskWriterEntry::initAndOpenFile to make files truncated.
-  for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
-      itr != diskWriterEntries.end(); ++itr) {
+  for(DiskWriterEntries::const_iterator itr = diskWriterEntries_.begin(),
+        eoi = diskWriterEntries_.end(); itr != eoi; ++itr) {
     openIfNot(*itr, &DiskWriterEntry::initAndOpenFile);
   }
 }
@@ -324,20 +296,22 @@ void MultiDiskAdaptor::openExistingFile()
 
 void MultiDiskAdaptor::closeFile()
 {
-  for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
-      itr != diskWriterEntries.end(); ++itr) {
-    (*itr)->closeFile();
-  }
+  std::for_each(diskWriterEntries_.begin(), diskWriterEntries_.end(),
+                mem_fun_sh(&DiskWriterEntry::closeFile));
 }
 
-static bool isInRange(const DiskWriterEntryHandle entry, off_t offset)
+namespace {
+bool isInRange(const DiskWriterEntryHandle entry, off_t offset)
 {
   return entry->getFileEntry()->getOffset() <= offset &&
-    (uint64_t)offset < entry->getFileEntry()->getOffset()+entry->getFileEntry()->getLength();
+    (uint64_t)offset <
+    entry->getFileEntry()->getOffset()+entry->getFileEntry()->getLength();
 }
+} // namespace
 
-static size_t calculateLength(const DiskWriterEntryHandle entry,
-                              off_t fileOffset, size_t rem)
+namespace {
+size_t calculateLength(const DiskWriterEntryHandle entry,
+                       off_t fileOffset, size_t rem)
 {
   size_t length;
   if(entry->getFileEntry()->getLength() < (uint64_t)fileOffset+rem) {
@@ -347,7 +321,9 @@ static size_t calculateLength(const DiskWriterEntryHandle entry,
   }
   return length;
 }
+} // namespace
 
+namespace {
 class OffsetCompare {
 public:
   bool operator()(off_t offset, const SharedHandle<DiskWriterEntry>& dwe)
@@ -355,9 +331,11 @@ public:
     return offset < dwe->getFileEntry()->getOffset();
   }
 };
+} // namespace
 
-static DiskWriterEntries::const_iterator
-findFirstDiskWriterEntry(const DiskWriterEntries& diskWriterEntries, off_t offset)
+namespace {
+DiskWriterEntries::const_iterator findFirstDiskWriterEntry
+(const DiskWriterEntries& diskWriterEntries, off_t offset)
 {
   DiskWriterEntries::const_iterator first =
     std::upper_bound(diskWriterEntries.begin(), diskWriterEntries.end(),
@@ -368,29 +346,34 @@ findFirstDiskWriterEntry(const DiskWriterEntries& diskWriterEntries, off_t offse
   // In case when offset is out-of-range
   if(!isInRange(*first, offset)) {
     throw DL_ABORT_EX
-      (StringFormat(EX_FILE_OFFSET_OUT_OF_RANGE,
-                    util::itos(offset, true).c_str()).str());
+      (fmt(EX_FILE_OFFSET_OUT_OF_RANGE,
+           util::itos(offset, true).c_str()));
   }
   return first;
 }
+} // namespace
 
-static void throwOnDiskWriterNotOpened(const SharedHandle<DiskWriterEntry>& e,
-                                       off_t offset)
+namespace {
+void throwOnDiskWriterNotOpened(const SharedHandle<DiskWriterEntry>& e,
+                                off_t offset)
 {
   throw DL_ABORT_EX
-    (StringFormat("DiskWriter for offset=%s, filename=%s is not opened.",
-                  util::itos(offset).c_str(),
-                  e->getFilePath().c_str()).str());  
+    (fmt("DiskWriter for offset=%s, filename=%s is not opened.",
+         util::itos(offset).c_str(),
+         e->getFilePath().c_str()));  
 }
+} // namespace
 
 void MultiDiskAdaptor::writeData(const unsigned char* data, size_t len,
                                  off_t offset)
 {
-  DiskWriterEntries::const_iterator first = findFirstDiskWriterEntry(diskWriterEntries, offset);
+  DiskWriterEntries::const_iterator first =
+    findFirstDiskWriterEntry(diskWriterEntries_, offset);
 
   size_t rem = len;
   off_t fileOffset = offset-(*first)->getFileEntry()->getOffset();
-  for(DiskWriterEntries::const_iterator i = first; i != diskWriterEntries.end(); ++i) {
+  for(DiskWriterEntries::const_iterator i = first,
+        eoi = diskWriterEntries_.end(); i != eoi; ++i) {
     size_t writeLength = calculateLength(*i, fileOffset, rem);
 
     openIfNot(*i, &DiskWriterEntry::openFile);
@@ -408,14 +391,17 @@ void MultiDiskAdaptor::writeData(const unsigned char* data, size_t len,
   }
 }
 
-ssize_t MultiDiskAdaptor::readData(unsigned char* data, size_t len, off_t offset)
+ssize_t MultiDiskAdaptor::readData
+(unsigned char* data, size_t len, off_t offset)
 {
-  DiskWriterEntries::const_iterator first = findFirstDiskWriterEntry(diskWriterEntries, offset);
+  DiskWriterEntries::const_iterator first =
+    findFirstDiskWriterEntry(diskWriterEntries_, offset);
 
   size_t rem = len;
   size_t totalReadLength = 0;
   off_t fileOffset = offset-(*first)->getFileEntry()->getOffset();
-  for(DiskWriterEntries::const_iterator i = first; i != diskWriterEntries.end(); ++i) {
+  for(DiskWriterEntries::const_iterator i = first,
+        eoi = diskWriterEntries_.end(); i != eoi; ++i) {
     size_t readLength = calculateLength(*i, fileOffset, rem);
 
     openIfNot(*i, &DiskWriterEntry::openFile);
@@ -437,60 +423,42 @@ ssize_t MultiDiskAdaptor::readData(unsigned char* data, size_t len, off_t offset
 
 bool MultiDiskAdaptor::fileExists()
 {
-  for(std::vector<SharedHandle<FileEntry> >::iterator i =
-        fileEntries.begin(); i != fileEntries.end(); ++i) {
-    if((*i)->exists()) {
-      return true;
-    }
-  }
-  return false;
+  return std::find_if(getFileEntries().begin(), getFileEntries().end(),
+                      mem_fun_sh(&FileEntry::exists)) !=
+    getFileEntries().end();
 }
 
 uint64_t MultiDiskAdaptor::size()
 {
   uint64_t size = 0;
-  for(std::vector<SharedHandle<FileEntry> >::iterator i =
-        fileEntries.begin(); i != fileEntries.end(); ++i) {
+  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
+        getFileEntries().begin(), eoi = getFileEntries().end(); i != eoi; ++i) {
     size += File((*i)->getPath()).size();
   }
   return size;
 }
 
-FileAllocationIteratorHandle MultiDiskAdaptor::fileAllocationIterator()
+SharedHandle<FileAllocationIterator> MultiDiskAdaptor::fileAllocationIterator()
 {
-  return SharedHandle<FileAllocationIterator>(new MultiFileAllocationIterator(this));
-}
-
-void MultiDiskAdaptor::enableDirectIO()
-{
-  for(DiskWriterEntries::const_iterator itr = diskWriterEntries.begin();
-      itr != diskWriterEntries.end(); ++itr) {
-    (*itr)->enableDirectIO();
-  }
-}
-
-void MultiDiskAdaptor::disableDirectIO()
-{
-  for(DiskWriterEntries::const_iterator itr = diskWriterEntries.begin();
-      itr != diskWriterEntries.end(); ++itr) {
-    (*itr)->disableDirectIO();
-  }
+  return SharedHandle<FileAllocationIterator>
+    (new MultiFileAllocationIterator(this));
 }
 
 void MultiDiskAdaptor::enableReadOnly()
 {
-  _readOnly = true;
+  readOnly_ = true;
 }
 
 void MultiDiskAdaptor::disableReadOnly()
 {
-  _readOnly = false;
+  readOnly_ = false;
 }
 
 void MultiDiskAdaptor::cutTrailingGarbage()
 {
   for(std::vector<SharedHandle<DiskWriterEntry> >::const_iterator i =
-        diskWriterEntries.begin(); i != diskWriterEntries.end(); ++i) {
+        diskWriterEntries_.begin(), eoi = diskWriterEntries_.end();
+      i != eoi; ++i) {
     uint64_t length = (*i)->getFileEntry()->getLength();
     if(File((*i)->getFilePath()).size() > length) {
       // We need open file before calling DiskWriter::truncate(uint64_t)
@@ -502,14 +470,14 @@ void MultiDiskAdaptor::cutTrailingGarbage()
 
 void MultiDiskAdaptor::setMaxOpenFiles(size_t maxOpenFiles)
 {
-  _maxOpenFiles = maxOpenFiles;
+  maxOpenFiles_ = maxOpenFiles;
 }
 
 size_t MultiDiskAdaptor::utime(const Time& actime, const Time& modtime)
 {
   size_t numOK = 0;
   for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-        fileEntries.begin(); i != fileEntries.end(); ++i) {
+        getFileEntries().begin(), eoi = getFileEntries().end(); i != eoi; ++i) {
     if((*i)->isRequested()) {
       File f((*i)->getPath());
       if(f.isFile() && f.utime(actime, modtime)) {
