@@ -22,8 +22,9 @@ class BitfieldManTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testAddNotFilter);
   CPPUNIT_TEST(testAddNotFilter_zeroLength);
   CPPUNIT_TEST(testAddNotFilter_overflow);
-  CPPUNIT_TEST(testGetSparceMissingUnusedIndex);
-  CPPUNIT_TEST(testGetSparceMissingUnusedIndex_setBit);
+  CPPUNIT_TEST(testGetSparseMissingUnusedIndex);
+  CPPUNIT_TEST(testGetSparseMissingUnusedIndex_setBit);
+  CPPUNIT_TEST(testGetSparseMissingUnusedIndex_withMinSplitSize);
   CPPUNIT_TEST(testIsBitSetOffsetRange);
   CPPUNIT_TEST(testGetMissingUnusedLength);
   CPPUNIT_TEST(testSetBitRange);
@@ -35,6 +36,7 @@ class BitfieldManTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testCountMissingBlock);
   CPPUNIT_TEST(testZeroLengthFilter);
   CPPUNIT_TEST(testGetFirstNMissingUnusedIndex);
+  CPPUNIT_TEST(testGetInorderMissingUnusedIndex);
   CPPUNIT_TEST_SUITE_END();
 public:
   void testGetBlockSize();
@@ -51,8 +53,9 @@ public:
   void testAddNotFilter();
   void testAddNotFilter_zeroLength();
   void testAddNotFilter_overflow();
-  void testGetSparceMissingUnusedIndex();
-  void testGetSparceMissingUnusedIndex_setBit();
+  void testGetSparseMissingUnusedIndex();
+  void testGetSparseMissingUnusedIndex_setBit();
+  void testGetSparseMissingUnusedIndex_withMinSplitSize();
   void testIsBitSetOffsetRange();
   void testGetMissingUnusedLength();
   void testSetBitRange();
@@ -60,6 +63,7 @@ public:
   void testCountMissingBlock();
   void testZeroLengthFilter();
   void testGetFirstNMissingUnusedIndex();
+  void testGetInorderMissingUnusedIndex();
 };
 
 
@@ -210,7 +214,7 @@ void BitfieldManTest::testFilter()
   std::vector<size_t> out;
   CPPUNIT_ASSERT_EQUAL((size_t)6, btman.getFirstNMissingUnusedIndex(out, 32));
   const size_t ans[] = { 2, 3, 4, 5, 6, 7 };
-  for(size_t i = 0; i < arrayLength(ans); ++i) {
+  for(size_t i = 0; i < A2_ARRAY_LEN(ans); ++i) {
     CPPUNIT_ASSERT_EQUAL(ans[i], out[i]);
   }
   CPPUNIT_ASSERT_EQUAL((uint64_t)12ULL, btman.getFilteredTotalLength());
@@ -275,103 +279,138 @@ void BitfieldManTest::testAddNotFilter_overflow() {
 }
 
 // TODO1.5 add test using ignoreBitfield
-void BitfieldManTest::testGetSparceMissingUnusedIndex() {
+void BitfieldManTest::testGetSparseMissingUnusedIndex() {
   BitfieldMan bitfield(1024*1024, 10*1024*1024);
   const size_t length = 2;
   unsigned char ignoreBitfield[length];
   memset(ignoreBitfield, 0, sizeof(ignoreBitfield));
+  size_t minSplitSize = 1024*1024;
   size_t index;
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)0, index);
   bitfield.setUseBit(0);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)5, index);
   bitfield.setUseBit(5);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)3, index);
   bitfield.setUseBit(3);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)8, index);
   bitfield.setUseBit(8);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)2, index);
   bitfield.setUseBit(2);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
-                                                      ignoreBitfield, length));
-  CPPUNIT_ASSERT_EQUAL((size_t)7, index);
-  bitfield.setUseBit(7);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)1, index);
   bitfield.setUseBit(1);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)4, index);
   bitfield.setUseBit(4);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
+                                                      ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)7, index);
+  bitfield.setUseBit(7);
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)6, index);
   bitfield.setUseBit(6);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)9, index);
   bitfield.setUseBit(9);
-  CPPUNIT_ASSERT(!bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(!bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                        ignoreBitfield, length));
 }
 
-void BitfieldManTest::testGetSparceMissingUnusedIndex_setBit() {
+void BitfieldManTest::testGetSparseMissingUnusedIndex_setBit() {
   BitfieldMan bitfield(1024*1024, 10*1024*1024);
   const size_t length = 2;
   unsigned char ignoreBitfield[length];
   memset(ignoreBitfield, 0, sizeof(ignoreBitfield));
+  size_t minSplitSize = 1024*1024;
   size_t index;
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)0, index);
   bitfield.setBit(0);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)1, index);
   bitfield.setBit(1);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)2, index);
   bitfield.setBit(2);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)3, index);
   bitfield.setBit(3);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)4, index);
   bitfield.setBit(4);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)5, index);
   bitfield.setBit(5);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)6, index);
   bitfield.setBit(6);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)7, index);
   bitfield.setBit(7);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)8, index);
   bitfield.setBit(8);
-  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                       ignoreBitfield, length));
   CPPUNIT_ASSERT_EQUAL((size_t)9, index);
   bitfield.setBit(9);
-  CPPUNIT_ASSERT(!bitfield.getSparseMissingUnusedIndex(index,
+  CPPUNIT_ASSERT(!bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
+                                                       ignoreBitfield, length));
+}
+
+void BitfieldManTest::testGetSparseMissingUnusedIndex_withMinSplitSize()
+{
+  BitfieldMan bitfield(1024*1024, 10*1024*1024);
+  const size_t length = 2;
+  unsigned char ignoreBitfield[length];
+  memset(ignoreBitfield, 0, sizeof(ignoreBitfield));
+  size_t minSplitSize = 2*1024*1024;
+  size_t index;
+  bitfield.setUseBit(1);
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
+                                                      ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)6, index);
+  bitfield.setBit(6);
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
+                                                      ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)7, index);
+  bitfield.setUseBit(7);
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
+                                                      ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)4, index);
+  bitfield.setBit(4);
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
+                                                      ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)0, index);
+  bitfield.setBit(0);
+  CPPUNIT_ASSERT(bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
+                                                      ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)5, index);
+  bitfield.setBit(5);  
+  CPPUNIT_ASSERT(!bitfield.getSparseMissingUnusedIndex(index, minSplitSize,
                                                        ignoreBitfield, length));
 }
 
@@ -603,4 +642,68 @@ void BitfieldManTest::testGetFirstNMissingUnusedIndex()
   CPPUNIT_ASSERT_EQUAL((size_t)9, out[0]);
 }
 
+void BitfieldManTest::testGetInorderMissingUnusedIndex()
+{
+  BitfieldMan bt(1024, 1024*20);
+  const size_t length = 3;
+  unsigned char ignoreBitfield[length];
+  memset(ignoreBitfield, 0, sizeof(ignoreBitfield));
+  size_t minSplitSize = 1024;
+  size_t index;
+  // 00000|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)0, index);
+  bt.setUseBit(0);
+  // 10000|00000|00000|00000
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)1, index);
+  minSplitSize = 1024*2;
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)2, index);
+  bt.unsetUseBit(0);
+  bt.setBit(0);
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)1, index);
+  bt.setAllBit();
+  bt.unsetBit(10);
+  // 11111|11111|01111|11111
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)10, index);
+  bt.setUseBit(10);
+  CPPUNIT_ASSERT
+    (!bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  bt.unsetUseBit(10);
+  bt.setAllBit();
+  // 11111|11111|11111|11111
+  CPPUNIT_ASSERT
+    (!bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  bt.clearAllBit();
+  // 00000|00000|00000|00000
+  for(int i = 0; i <= 1; ++i) {
+    bitfield::flipBit(ignoreBitfield, length, i);
+  }
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)2, index);
+  bt.addFilter(1024*3, 1024*3);
+  bt.enableFilter();
+  CPPUNIT_ASSERT
+    (bt.getInorderMissingUnusedIndex
+     (index, minSplitSize, ignoreBitfield, length));
+  CPPUNIT_ASSERT_EQUAL((size_t)3, index);
+}
+
 } // namespace aria2
